package com.ubains.demorx356x;

import androidx.appcompat.app.AppCompatActivity;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.os.yx.IYxInstallListener;
import android.os.yx.YxDeviceManager;
import android.provider.Settings;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import com.ubains.demorx356x.databinding.ActivityMainBinding;

import org.apache.commons.net.ntp.NTPUDPClient;
import org.apache.commons.net.ntp.TimeInfo;
import org.apache.commons.net.ntp.TimeStamp;

import java.io.File;
import java.net.InetAddress;
import java.util.Calendar;
import java.util.Date;

public class MainActivity extends AppCompatActivity {
    private static final String TAG = "MainActivity";

    final YxDeviceManager yxDeviceManager = YxDeviceManager.getInstance(this);
    TextView tvLog;
    ActivityMainBinding binding;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        tvLog = binding.tvLog;

        binding.switchLightBlue.setOnCheckedChangeListener((compoundButton, b) -> {
            if (b) {
                YxDeviceManager.getInstance(getApplication()).setOemFunc("io21");
            } else {
                YxDeviceManager.getInstance(getApplication()).setOemFunc("io20");
            }
        });
        binding.switchLightRed.setOnCheckedChangeListener((compoundButton, b) -> {
            if (b) {
                YxDeviceManager.getInstance(getApplication()).setOemFunc("io31");
            } else {
                YxDeviceManager.getInstance(getApplication()).setOemFunc("io30");
            }
        });
        binding.switchLightGreen.setOnCheckedChangeListener((compoundButton, b) -> {
            if (b) {
                YxDeviceManager.getInstance(getApplication()).setOemFunc("io41");
            } else {
                YxDeviceManager.getInstance(getApplication()).setOemFunc("io40");
            }
        });

        binding.switchRelay.setOnCheckedChangeListener((compoundButton, b) -> {
            if (b) {
                YxDeviceManager.getInstance(getApplication()).setOemFunc("io51");
            } else {
                YxDeviceManager.getInstance(getApplication()).setOemFunc("io50");
            }
        });


        //注册广播监听WIFI状态变化
        registerWifiReceiver();
    }

    private void registerWifiReceiver() {
        // 创建Wifi状态变化的IntentFilter
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(WifiManager.WIFI_STATE_CHANGED_ACTION);

        // 注册BroadcastReceiver
        registerReceiver(wifiStateReceiver, intentFilter);
    }

    /**
     * 打印系统信息
     *
     * @param view
     */
    public void printSystemInfo(View view) {
        final StringBuilder info = new StringBuilder();
        info.append("设备型号:").append(yxDeviceManager.getAndroidModle()).append("\n");
        info.append("亮度:").append(yxDeviceManager.getBrightness()).append("\n");
        info.append("MAC:").append(yxDeviceManager.getDeviceMacaddress()).append("\n");
        info.append("以太网状态:").append(yxDeviceManager.getEthStatus()).append("\n");
        info.append("IP Address:").append(yxDeviceManager.getIpAddress()).append("\n");
        info.append("是否Root:").append(yxDeviceManager.getRootStatus()).append("\n");
        info.append("系统亮度:").append(yxDeviceManager.getSystemBrightness()).append("\n");
        info.append("IMEI:").append(yxDeviceManager.getTelephonyImei()).append("\n");
        info.append("DNS1:").append(yxDeviceManager.getEthDns1()).append("\n");
        info.append("DNS2:").append(yxDeviceManager.getEthDns2()).append("\n");
        info.append("AndroidVersion:").append(yxDeviceManager.getAndroidVersion()).append("\n");
        info.append("ApiVersion:").append(yxDeviceManager.getApiVersion()).append("\n");
        info.append("网关:").append(yxDeviceManager.getGateway()).append("\n");
        info.append("内核版本:").append(yxDeviceManager.getKernelVersion()).append("\n");
        info.append("运行内存:").append(yxDeviceManager.getRunningMemory()).append("\n");
        info.append("以太网模式:").append(yxDeviceManager.getEthMode()).append("\n");
        info.append("屏幕方向:").append(yxDeviceManager.getScreenRotation()).append("\n");
        info.append("序列号:").append(yxDeviceManager.getSerialno()).append("\n");
        info.append("内部存储大小:").append(yxDeviceManager.getInternalStorageMemory()).append("\n");


        //读取WIFI的默认策略
        int wifiSleepPolicy = Settings.System.getInt(getContentResolver(),
                android.provider.Settings.System.WIFI_SLEEP_POLICY,
                Settings.System.WIFI_SLEEP_POLICY_DEFAULT);
        System.out.println("---> 修改前的Wifi休眠策略值 WIFI_SLEEP_POLICY=" + wifiSleepPolicy);

        info.append("WIFI_SLEEP_POLICY:").append(wifiSleepPolicy).append("\n");

        tvLog.setText(info.toString());
    }

    /**
     * 改变系统时间
     *
     * @param view
     */
    public void setSysTime(View view) {
        final Calendar instance = Calendar.getInstance();
        instance.set(Calendar.HOUR_OF_DAY, 9);
        instance.set(Calendar.MINUTE, 0);
        instance.set(Calendar.SECOND, 0);
        //设置时间
        yxDeviceManager.setSystemTime(instance.getTimeInMillis());
    }

    /**
     * NTP 时间同步
     *
     * @param view
     */
    public void ntpSyncTime(View view) {
        view.setEnabled(false);
        new Thread(() -> {
            try {
                runOnUiThread(() -> {
                    tvLog.setText("开始同步时间，Ntp地址：ntp1.aliyun.com\n");
                });
                //url为ntp地址,如:ntp1.aliyun.com
                final InetAddress inetAddress = InetAddress.getByName("ntp1.aliyun.com");
                final NTPUDPClient ntpudpClient = new NTPUDPClient();
                //设置超时时间
                ntpudpClient.setDefaultTimeout(10000);
                //获取时间，为防止堵塞UI需要在其他线程中调用
                final TimeInfo timeinfo = ntpudpClient.getTime(inetAddress);
                final TimeStamp transmitTimeStamp = timeinfo.getMessage().getTransmitTimeStamp();
                runOnUiThread(() -> {
                    tvLog.append("同步时间成功，" + transmitTimeStamp.getDate());
                });
                //设置时间
                yxDeviceManager.setSystemTime(transmitTimeStamp.getTime());
            } catch (Exception e) {
                runOnUiThread(() -> {
                    tvLog.setText("NTP同步时间失败：" + e.getMessage());
                });
            } finally {
                runOnUiThread(() -> {
                    view.setEnabled(true);
                });
            }
        }).start();
    }

    public void devicePowerOff(View view) {
        yxDeviceManager.shutDownNow();
    }

    public void deviceReboot(View view) {
        yxDeviceManager.rebootNow();
    }

    public void openNavBar(View view) {
        yxDeviceManager.setNavBarNew(true);
    }

    public void closeNavBar(View view) {
        yxDeviceManager.setNavBarNew(false);
    }

    public void openStatusBar(View view) {
        yxDeviceManager.setStaBarNew(true);
    }

    public void closeStatusBar(View view) {
        yxDeviceManager.setStaBarNew(false);
    }

    public void sliceInstall(View view) {
        File apkFile = new File("/sdcard/update.apk");
        if (!apkFile.exists()) {
            tvLog.setText("/sdcard/update.apk 文件不存在");
            return;
        }
        yxDeviceManager.silentInstallApk("/sdcard/update.apk");
    }

    public void sliceUninstall(View view) {
        yxDeviceManager.unInstallApk("org.videolan.vlc");
    }

    public void turnOffScreen(View view) {
        yxDeviceManager.setScreenKeyGuard(false);
    }

    public void setWifiSleepPolicyNever(View view) {
        Settings.System.putInt(getContentResolver(),
                android.provider.Settings.System.WIFI_SLEEP_POLICY,
                Settings.System.WIFI_SLEEP_POLICY_NEVER);
        tvLog.setText("OK!");
    }


    private final BroadcastReceiver wifiStateReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int wifiState = intent.getIntExtra(WifiManager.EXTRA_WIFI_STATE, WifiManager.WIFI_STATE_UNKNOWN);
            switch (wifiState) {
                case WifiManager.WIFI_STATE_ENABLED:
                    Log.d(TAG, "onReceive: Wifi已启用");
                    break;
                case WifiManager.WIFI_STATE_DISABLED:
                    Log.d(TAG, "onReceive: Wifi已禁用");
                    break;
                case WifiManager.WIFI_STATE_UNKNOWN:
                    Log.d(TAG, "onReceive: Wifi状态未知");
                    break;
            }
        }
    };


}